package org.telegram.telegrambots.updatesreceivers;

import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;
import lombok.extern.slf4j.Slf4j;
import org.telegram.telegrambots.Constants;
import org.telegram.telegrambots.meta.api.methods.BotApiMethod;
import org.telegram.telegrambots.meta.api.objects.Update;
import org.telegram.telegrambots.meta.exceptions.TelegramApiValidationException;
import org.telegram.telegrambots.meta.generics.WebhookBot;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Rest api to for webhook callback function
 */
@Path(Constants.WEBHOOK_URL_PATH)
@Slf4j
public class RestApi {
    private final ConcurrentHashMap<String, WebhookBot> callbacks = new ConcurrentHashMap<>();

    public RestApi() {
    }

    public void registerCallback(WebhookBot callback) {
        if (!callbacks.containsKey(callback.getBotPath())) {
            callbacks.put(callback.getBotPath(), callback);
        }
    }

    @POST
    @Path("/{botPath}")
    @Consumes(MediaType.APPLICATION_JSON)
    @Produces(MediaType.APPLICATION_JSON)
    public Response updateReceived(@PathParam("botPath") String botPath, Update update) {
        if (callbacks.containsKey(botPath)) {
            try {
                BotApiMethod<?> response = callbacks.get(botPath).onWebhookUpdateReceived(update);
                if (response != null) {
                    response.validate();
                }
                return Response.ok(response).build();
            } catch (TooManyRequestsException e) {
                if (e.getProcessedRequestsCount().isPresent()) {
                    return Response
                        .status(Status.TOO_MANY_REQUESTS)
                        .header("X-Processed-Requests", e.getProcessedRequestsCount().getAsInt())
                        .build();
                } else {
                    return Response.status(Status.TOO_MANY_REQUESTS).build();
                }
            } catch (TelegramApiValidationException e) {
                log.error(e.getLocalizedMessage(), e);
                return Response.serverError().build();
            }
        }

        return Response.status(Response.Status.NOT_FOUND).build();
    }

    @POST
    @Path("/{botPath}/multi")
    @Consumes(MediaType.APPLICATION_JSON)
    @Produces(MediaType.APPLICATION_JSON)
    public Response updatesReceived(@PathParam("botPath") String botPath, List<Update> updates) {
        if (callbacks.containsKey(botPath)) {
            try {
                List<BotApiMethod<?>> responses = callbacks.get(botPath).onWebhookUpdatesReceived(updates);
                if (responses != null) {
                    for (BotApiMethod<?> response : responses) {
                        if (response != null) {
                            response.validate();
                        }
                    }
                }
                return Response.ok(responses).build();
            } catch (TooManyRequestsException e) {
                if (e.getProcessedRequestsCount().isPresent()) {
                    return Response
                        .status(Status.TOO_MANY_REQUESTS)
                        .header("X-Processed-Requests", e.getProcessedRequestsCount().getAsInt())
                        .build();
                } else {
                    return Response.status(Status.TOO_MANY_REQUESTS).build();
                }
            } catch (TelegramApiValidationException e) {
                log.error(e.getLocalizedMessage(), e);
                return Response.serverError().build();
            }
        }

        return Response.status(Response.Status.NOT_FOUND).build();
    }

    @GET
    @Path("/{botPath}")
    @Produces(MediaType.APPLICATION_JSON)
    public String testReceived(@PathParam("botPath") String botPath) {
        if (callbacks.containsKey(botPath)) {
            return "Hi there " + botPath + "!";
        } else {
            return "Callback not found for " + botPath;
        }
    }
}
